C***********************************************************************
C    Module:  iopol.f
C 
C    Copyright (C) 2000 Mark Drela, Harold Youngren
C 
C    This program is free software; you can redistribute it and/or modify
C    it under the terms of the GNU General Public License as published by
C    the Free Software Foundation; either version 2 of the License, or
C    (at your option) any later version.
C
C    This program is distributed in the hope that it will be useful,
C    but WITHOUT ANY WARRANTY; without even the implied warranty of
C    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C    GNU General Public License for more details.
C
C    You should have received a copy of the GNU General Public License
C    along with this program; if not, write to the Free Software
C    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
C***********************************************************************

      SUBROUTINE POLREAD(LU,FNPOL,ERROR,
     &                   NAX,NA,CPOL, 
     &                   REYN1,MACH1,ACRIT,XTRIP,
     &                   NAME, IRETYP,IMATYP,
     &                   ISX,NBL,CPOLSD,
     &                   CODE,VERSION )
      INCLUDE 'PINDEX.INC'
      CHARACTER*(*) FNPOL, NAME
      LOGICAL ERROR
      CHARACTER*(*) CODE
      REAL CPOL(NAX,IPTOT), CPOLSD(NAX,ISX,JPTOT)
      REAL MACH1, XTRIP(ISX)
C--------------------------------------------------------
C     Reads in polar save file
C
C  Input:
C     LU      logical unit to use for reading
C     FNPOL   name of polar file to be read,
C               if FNPOL(1:1).eq.' ', unit LU will be read
C               if it is already open
C     NAX     polar point array dimension
C     ISX     airfoil side array dimension
C
C  Output:
C     ERROR   T if a READ error occurred
C     NA      number polar points
C     CPOL    polar coefficients and parameters
C     REYN1   Reynolds number for CL=1
C     MACH1   Mach number for CL=1
C     ACRIT   Critical amplification ratio
C     XTRIP   Trip locations
C     NAME    airfoil name string
C     IRETYP  flag giving type of Re variation with CL
C     IMATYP  flag giving type of Ma variation with CL
C     NBL     number of airfoil elements
C     CPOLSD  airfoil side-related parameters
C     CODE    code used to compute polar
C     VERSION code version
C--------------------------------------------------------
      CHARACTER*128 LINE
      CHARACTER*1 DUMMY
      REAL RINP(IPTOT)
C
      LOGICAL LOPEN, LHEAD
C
C
      ERROR = .FALSE.
      LHEAD = .TRUE.
C
      NA = 0
      NBL = 1
C
      KCH = 0
      KMC = 0
C
C---- BUG:  moved out of 500 loop below   MD  24 Mar 01
      IRETYP = 1
      IMATYP = 1
C
C---- do we have to open the file?
      LOPEN = FNPOL .NE. ' '
C
      IF(LOPEN) OPEN(LU,FILE=FNPOL,STATUS='OLD',ERR=90)
C
C=============================================================
C---- start data reading loop
 500  CONTINUE
      READ(LU,1000,END=80) LINE
      IF(LINE.EQ.' ') GO TO 500
C
      IF(LHEAD) THEN
C----- parse to get header info
C
C--------------------------------------------
       K = INDEX(LINE,'Version')
       IF(K.NE.0) THEN
C------ code,version line
        DO K1=1, 128
          IF(LINE(K1:K1).NE.' ') GO TO 10
        ENDDO
C
 10     CONTINUE
        IF(K.GT.K1) THEN
         CODE = LINE(K1:K-1)
         READ(LINE(K+7:128),*,ERR=11) VERSION
        ENDIF
 11     CONTINUE
       ENDIF
C
C--------------------------------------------
       KF = INDEX(LINE,'for:')
       IF(KF.NE.0) THEN
C------ airfoil name line
        NAME = LINE(KF+5:128)
       ENDIF
C
C--------------------------------------------
       KE = INDEX(LINE,'elements')
       IF(KE.GT.0) THEN
C------ element-number line
        READ(LINE(KE-4:KE-1),*,ERR=60) NBL
C------ truncate name line to eliminate elements #
        NAME = LINE(KF+5:KE-4)
C
        IF(2*NBL .GT. ISX) THEN
         NBL = ISX/2
         WRITE(*,*) 
     &   'POLREAD: Number of elements set to array limit', NBL
        ENDIF
       ENDIF
C
C--------------------------------------------
       KR = INDEX(LINE,'Reynolds number')
       KM = INDEX(LINE,'Mach number')
C
       IF(KR.NE.0) THEN
C------ Re-type line
        IF(KM.GT.KR) THEN
         KEND = KM-1
        ELSE
         KEND = 128
        ENDIF
        IF    (INDEX(LINE(KR:KEND),'fixed').NE.0) THEN
         IRETYP = 1
        ELSEIF(INDEX(LINE(KR:KEND),'1/sqrt(CL)').NE.0) THEN
         IRETYP = 2
        ELSEIF(INDEX(LINE(KR:KEND),'1/CL').NE.0) THEN
         IRETYP = 3
        ENDIF
       ENDIF
C
       IF(KM.NE.0) THEN
C------ Ma-type line
        IF(KR.GT.KM) THEN
         KEND = KR-1
        ELSE
         KEND = 128
        ENDIF
        IF    (INDEX(LINE(KM:KEND),'fixed').NE.0) THEN
         IMATYP = 1
        ELSEIF(INDEX(LINE(KM:KEND),'1/sqrt(CL)').NE.0) THEN
         IMATYP = 2
        ELSEIF(INDEX(LINE(KM:KEND),'1/CL').NE.0) THEN
         IMATYP = 3
        ENDIF
       ENDIF
C
C--------------------------------------------
C---- find specified BL trip location
       K = INDEX(LINE,'xtrf')
       IF(K.NE.0) THEN
C------ new style xtrip line
        KT = INDEX(LINE,'(top)')
        KB = INDEX(LINE,'(bottom)')
        KE = INDEX(LINE,'element ')
C--- check for old style trip line 
        KS = INDEX(LINE,'(suc')
        KP = INDEX(LINE,'(pre')
C
        IF(KE.NE.0) THEN
         READ(LINE(KE+7:KE+12),*,ERR=21) N
        ELSE
         N = 1
        ENDIF
        IF(N.LE.NBL) THEN
         IS1 = 2*N-1
         IS2 = 2*N
         XTRIP(IS1) = 1.0
         XTRIP(IS2) = 1.0
         IF(KT.GT.0)  READ(LINE(K+6:KT-1) ,*,ERR=21) XTRIP(IS1)
         IF(KB.GT.KT) READ(LINE(KT+5:KB-1),*,ERR=21) XTRIP(IS2)
         IF(KS.GT.0)  READ(LINE(K+6:KS-1) ,*,ERR=21) XTRIP(IS1)
         IF(KP.GT.KS) READ(LINE(KS+5:KP-1),*,ERR=21) XTRIP(IS2)
        ENDIF
 21     CONTINUE
       ENDIF
C
C--------------------------------------------
       K = INDEX(LINE,'Mach =')
       IF(K.NE.0) THEN
        READ(LINE(K+6:128),*,ERR=31) MACH1
 31     CONTINUE
       ENDIF
C
C--------------------------------------------
       K = INDEX(LINE,'Re =')
       IF(K.NE.0) THEN
        READ(LINE(K+4:128),*,ERR=32) REYN1
        REYN1 = REYN1 * 1.0E6
 32     CONTINUE
       ENDIF
C
C--------------------------------------------
       K = INDEX(LINE,'Ncrit =')
       IF(K.NE.0) THEN
        READ(LINE(K+7:128),*,ERR=33) ACRIT
 33     CONTINUE
       ENDIF
C
C--------------------------------------------
C---- process line with data labels, check for extra data params (hinge,cpmin) 
       K = INDEX(LINE,'Chinge')
       IF(K.NE.0) KCH = K
C
       K = INDEX(LINE,'Cpmin')
       IF(K.NE.0) KMC = K
C
C--------------------------------------------
       IF(INDEX(LINE,'-----').NE.0) THEN
        LHEAD = .FALSE.
       ENDIF
C
C--------------------------------------------------------------
      ELSE
C---- read polar data lines
C
C     One problem here is to read all the relevant formats, XFOIL and the 
C     myriad MSES formats for polars.  MSES polars do not contain the hinge
C     moment or Cpmin data but may contain sensitivity data dCL/da,dCD/da,
C     dCL/dM,dCD/dM located after the basic 5 alfa,CL,CD,CDp/w,CM.  There 
C     are 2*NBL transition locations to read, located after the basic 5 and
C     any sensitivity data.  The hinge and cpmin data are optionally at the 
C     end of the line in either order.
C
       IA = NA + 1
C
       NINP = IPTOT
       CALL GETFLT(LINE,RINP,NINP,ERROR)
       IF(ERROR) GO TO 90
C
       CPOL(IA,IAL) = RINP(1)
       CPOL(IA,ICL) = RINP(2)
       CPOL(IA,ICD) = RINP(3)
       CPOL(IA,ICW) = RINP(4)
       CPOL(IA,ICM) = RINP(5)
C
C---- check for hinge moment data in last two columns
       IITR = NINP
       IF(KCH.GT.0) THEN
        IF(KMC.EQ.0 .OR. KMC.GT.KCH) THEN
         CPOL(IA,ICH) = RINP(NINP-1)
        ELSE
         CPOL(IA,ICH) = RINP(NINP)
        ENDIF
        IITR = IITR - 1
       ELSE
        CPOL(IA,ICH) = 0.
       ENDIF
C
C---- check for Cpmin data in last two columns
       IF(KMC.GT.0) THEN
        IF(KCH.EQ.0 .OR. KCH.GT.KMC) THEN
         CPOL(IA,IMC) = RINP(NINP-1)
        ELSE
         CPOL(IA,IMC) = RINP(NINP)
        ENDIF
        IITR = IITR - 1
       ELSE
        CPOL(IA,IMC) = 0. 
       ENDIF
C
C---- assign trip locations from next previous 2*NBL values on line
       IF(IITR-5.GE.2*NBL) THEN
        DO IS = 1, 2*NBL
          II = IITR-2*NBL + IS
          CPOLSD(IA,IS,JTN) = RINP(II)
        ENDDO
       ELSE
        WRITE(*,*) '*** Error reading transition locations'
        DO IS = 1, 2*NBL
          CPOLSD(IA,IS,JTN) = 1.0
        END DO
       ENDIF
C
C---- process and save RE-CL and MA-CL dependent data
       ACL = MAX( CPOL(IA,ICL) , 0.001 )
       IF    (IRETYP.EQ.1) THEN
        CPOL(IA,IRE) = REYN1
       ELSEIF(IRETYP.EQ.2) THEN
        CPOL(IA,IRE) = REYN1/SQRT(ACL) 
       ELSE
        CPOL(IA,IRE) = REYN1/ACL
       ENDIF
       IF    (IMATYP.EQ.1) THEN
        CPOL(IA,IMA) = MACH1
       ELSEIF(IMATYP.EQ.2) THEN
        CPOL(IA,IMA) = MACH1/SQRT(ACL)
       ELSE
        CPOL(IA,IMA) = MACH1/ACL
       ENDIF
       CPOL(IA,INC) = ACRIT
C
C---- save trip data 
       DO IS = 1, 2*NBL
         CPOLSD(IA,IS,JTP) = XTRIP(IS)
       ENDDO
C
       NA = IA
      ENDIF
C
 60   CONTINUE
C---- go read next line
      GO TO 500
C=============================================================
C
 80   CONTINUE
C---- if file was opened here, then close it
      IF(LOPEN) CLOSE(LU)
      RETURN
C
 90   CONTINUE
      IF(LOPEN) CLOSE(LU)
      ERROR = .TRUE.
      RETURN
C
C..........................................
 1000 FORMAT(A)
 1005 FORMAT(A16,12X,F10.0)
 1010 FORMAT(22X,A32)
 1015 FORMAT(2I2)
 1020 FORMAT( 8X,F7.3,10X,F9.3)
 1030 FORMAT( 8X,F7.3,10X,F9.3,17X,F7.3)
      END ! POLREAD


      SUBROUTINE POLWRIT(LU,FNPOL,ERROR, LHEAD,
     &                   NAX, IA1,IA2, CPOL,
     &                   REYN1,MACH1,ACRIT,XTRIP,
     &                   NAME, IRETYP,IMATYP,
     &                   ISX,NBL,CPOLSD,
     &                   CODE,VERSION, LQUERY, LCMINP,LHMOMP )
      INCLUDE 'PINDEX.INC'
      CHARACTER*(*) FNPOL, NAME
      LOGICAL ERROR, LHEAD,LQUERY,LCMINP,LHMOMP
      CHARACTER*(*) CODE
      REAL CPOL(NAX,IPTOT), CPOLSD(NAX,ISX,JPTOT)
      REAL MACH1, XTRIP(ISX)
C--------------------------------------------------------
C     Writes polar save file
C
C  Input:
C     LU      logical unit to use for writing
C     FNPOL   name of polar file to be read,
C               if FNPOL(1:1).eq.' ', unit LU is assumed 
C               to be already open
C     NAX     polar point array dimension
C     ISX     airfoil side array dimension
C     IA1,IA2 only polar points IA1..IA2 are written
C     CPOL    polar coefficients and parameters
C     REYN1   Reynolds number for CL=1
C     MACH1   Mach number for CL=1
C     ACRIT   Critical amplification ratio
C     XTRIP   Trip locations
C     NAME    airfoil name string
C     IRETYP  flag giving type of Re variation with CL
C     IMATYP  flag giving type of Ma variation with CL
C     NBL     number of airfoil elements
C     CPOLSD  airfoil side-related parameters
C     LHEAD   T if header and column label are to be written
C     CODE    code used to compute polar
C     VERSION code version
C     LQUERY  if T, asks permission to overwrite existing file
C     LCMINP  if T, minimum Cp column will be written
C     LHMOMP  if T, hinge moment coefficient will be written
C
C  Output:
C     ERROR   T if a OPER or WRITE error occurred
C--------------------------------------------------------
      CHARACTER*29 LINE1, LINE2
      CHARACTER*80 LINEL
      CHARACTER*1 ANS
      LOGICAL LOPEN
C
      ERROR = .FALSE.
C
C---- do we have to open the file?
      LOPEN = FNPOL .NE. ' '
C
      IF(LOPEN) THEN
       OPEN(LU,FILE=FNPOL,STATUS='OLD',ERR=20)
C
       IF(LQUERY) THEN
        WRITE(*,*)
        WRITE(*,*) 'Output file exists.  Overwrite?  Y'
        READ(*,1000) ANS
C
        IF(INDEX('Nn',ANS).EQ.0) GO TO 22
C
        CLOSE(LU)
        WRITE(*,*) 'Polar file not saved'
        RETURN
       ENDIF
C
 20    OPEN(LU,FILE=FNPOL,STATUS='UNKNOWN',ERR=90)
 22    REWIND(LU)
      ENDIF
C
      IF(LHEAD) THEN
       WRITE(LU,*) ' '
       WRITE(LU,8000) CODE, VERSION
       WRITE(LU,*) ' '
       IF(NBL.EQ.1) THEN
        WRITE(LU,9001) NAME
       ELSE
        WRITE(LU,9002) NAME, NBL
       ENDIF
       WRITE(LU,*) ' '
C
       IF(IRETYP.EQ.1) LINE1 = ' Reynolds number fixed       '
       IF(IRETYP.EQ.2) LINE1 = ' Reynolds number ~ 1/sqrt(CL)'
       IF(IRETYP.EQ.3) LINE1 = ' Reynolds number ~ 1/CL      '
       IF(IMATYP.EQ.1) LINE2 = '   Mach number fixed         '
       IF(IMATYP.EQ.2) LINE2 = '   Mach number ~ 1/sqrt(CL)  '
       IF(IMATYP.EQ.3) LINE2 = '   Mach number ~ 1/CL        '
       WRITE(LU,9005) IRETYP, IMATYP, LINE1, LINE2
C
       WRITE(LU,*) ' '
       DO N = 1, NBL
         IS1 = 2*N-1
         IS2 = 2*N
         IF(NBL.EQ.1) THEN
          WRITE(LU,9011) XTRIP(IS1), XTRIP(IS2)
         ELSE
          WRITE(LU,9012) XTRIP(IS1), XTRIP(IS2), N
         ENDIF
       ENDDO
       WRITE(LU,9015) MACH1, REYN1/1.0E6, ACRIT
       WRITE(LU,*) ' '
       LINEL = 
     & '  alpha     CL        CD       CDp       CM    Top_Xtr Bot_Xtr'
CCC     1234567890123456789012345678901234567890123456789012345678901234567890
       K = 62
C
       IF(LCMINP) THEN
        LINEL(K+1:K+9) = '  Cpmin  '
        K = K+9
       ENDIF
       IF(LHMOMP) THEN
        LINEL(K+1:K+9) = '  Chinge '
        K = K+9
       ENDIF
C
       WRITE(LU,1000) LINEL(1:K)
C
C
       LINEL =
     & ' ------- -------- --------- --------- -------- ------- -------'
CCC       3.453   1.3750   0.00921   0.00512  -0.1450  0.9231  0.5382
CCC       3.453   1.3750   0.00921   0.00213  -0.1450  0.9231  0.5382
       K = 62
C
       IF(LCMINP) THEN
        LINEL(K+1:K+9) = ' ------- '
        K = K+9
       ENDIF
       IF(LHMOMP) THEN
        LINEL(K+1:K+9) = ' ------- '
        K = K+9
       ENDIF
C
       WRITE(LU,1000) LINEL(1:K)
      ENDIF
C
      DO 40 IA = IA1, IA2
        WRITE(LINEL,9100) CPOL(IA,IAL),CPOL(IA,ICL),
     &                    CPOL(IA,ICD),CPOL(IA,ICW),
     &                    CPOL(IA,ICM),
     &                    (CPOLSD(IA,IS,JTN), IS=1, 2)
        K = 62
C
        IF(LCMINP) THEN
         WRITE(LINEL(K+1:K+9),9100) CPOL(IA,IMC)
         K = K+9
        ENDIF
        IF(LHMOMP) THEN
         WRITE(LINEL(K+1:K+9),9100) CPOL(IA,ICH)
         K = K+9
        ENDIF
C
        WRITE(LU,1000) LINEL(1:K)
   40 CONTINUE
C
C---- if file was opened here, then close it
      IF(LOPEN) CLOSE(LU)
      RETURN
C
   90 CONTINUE
      ERROR = .TRUE.
      RETURN
C
C......................................................................
 1000 FORMAT(A)
 8000 FORMAT(7X,A,9X,'Version', F5.2)
 9001 FORMAT(1X,'Calculated polar for: ', A)
 9002 FORMAT(1X,'Calculated polar for: ', A, I4,' elements')
 9005 FORMAT(1X,I1,I2,2A29)
 9011 FORMAT(1X,
     &'xtrf = ',F7.3,' (top)    ',F9.3,' (bottom)  ')
 9012 FORMAT(1X,
     &'xtrf = ',F7.3,' (top)    ',F9.3,' (bottom)     element', I3)
 9015 FORMAT(1X,
     &'Mach = ',F7.3,5X,'Re = ',F9.3,' e 6',5X,'Ncrit = ',F7.3)
 9100 FORMAT(1X,F7.3,F9.4,2F10.5,F9.4,2F8.4  ,  F9.5)
CCC      3.453   1.3750   0.00921     0.500  -0.1450  0.9231  0.5382 -0.00942
CCC      3.453   1.3750   0.00921     0.500  -0.1450  0.9231  0.5382
      END



      SUBROUTINE POLREF(LU,FNREF,ERROR,
     &            NFX,NF,XYREF,LABREF )
      INCLUDE 'PINDEX.INC'
      CHARACTER*(*) FNREF,LABREF
      LOGICAL ERROR
      DIMENSION NF(4)
      DIMENSION XYREF(NFX,2,4)
C--------------------------------------------------------
C     Reads in polar reference data file
C
C  Input:
C     LU      logical unit to use for reading
C     FNREF   name of polar file to be read,
C               if FNREF(1:1).eq.' ', unit LU is assumed 
C               to be already open
C     NFX     polar point array dimension
C
C  Output:
C     ERROR      T if a READ error occurred
C     NF(.)      number of points in each data block
C     XYREF(...) reference polar data
C     LABREF(.)  reference polar label
C--------------------------------------------------------
      LOGICAL LOPEN
      CHARACTER*80 LINE
C
      ERROR = .FALSE.
      LOPEN = FNREF(1:1) .NE. ' '
      IF(LOPEN) OPEN(LU,FILE=FNREF,STATUS='OLD',ERR=900)
C
C---- try to read data label
      READ(LU,1000,END=900) LINE
 1000 FORMAT(A)
C
C---- set data label if present
      IF(LINE(1:1).EQ.'#') THEN
        LABREF = LINE(2:80)
      ELSE
        LABREF = ' '
        REWIND(LU)
      ENDIF
C
      DO 100 K=1, 4
        DO 10 I=1, NFX
          READ(LU,*,END=11,ERR=900) XYREF(I,1,K), XYREF(I,2,K)
          IF(XYREF(I,1,K) .EQ. 999.0) GO TO 11
   10   CONTINUE
   11   NF(K) = I-1
  100 CONTINUE
      IF(LOPEN) CLOSE(LU)
      RETURN
C
  900 CONTINUE
      ERROR = .TRUE.
C
      RETURN
      END ! POLREF
